@@ -19,7 +19,7 @@ our %EXPORT_TAGS =
 our @EXPORT_OK = ( @{$EXPORT_TAGS{'all'} } ) ;
 
 our @EXPORT = qw(DumpTree DumpTrees CreateChainingFilter);
-our $VERSION = '0.18' ;
+our $VERSION = '0.19' ;
 
 my $WIN32_CONSOLE ;
 
@@ -346,6 +346,7 @@ for (my $nodes_left = $#nodes_to_display ; $nodes_left >= 0 ; $nodes_left--)
 	my $tag = '' ;
 	my $element_value = '' ;
 	my $is_terminal_node = 0 ;
+	my $default_element_rendering  = '' ;
 	
 	for(ref $element)
 		{
@@ -354,12 +355,21 @@ for (my $nodes_left = $#nodes_to_display ; $nodes_left >= 0 ; $nodes_left--)
 			$is_terminal_node++ ;
 			$tag = 'S' ;
 			
-			my $value = defined $element ? $element : 'undef' ;
-			$element_address = $element_id ; # OK for terminal nodes
+			$element_address = $already_displayed_nodes->{NEXT_INDEX} ;
 			
+			my $value = defined $element ? $element : 'undef' ;
 			$element_value = "$value" ;
 			$element_value =~ s/\n/[\\n]/g ;
 			
+			if($setup->{QUOTE_VALUES} && defined $element)
+				{
+				$default_element_rendering = " = '$element_value'" ;
+				}
+			else
+				{
+				$default_element_rendering = " = $element_value" ;
+				}
+			
 			$perl_address = "$element_id" if($setup->{DISPLAY_PERL_ADDRESS}) ;
 			last ;
 			} ;
@@ -388,6 +398,7 @@ for (my $nodes_left = $#nodes_to_display ; $nodes_left >= 0 ; $nodes_left--)
 				
 			$tag = 'H' ;
 			$perl_address = "$element" if($setup->{DISPLAY_PERL_ADDRESS}) ;
+			$default_element_rendering = ' (no elements)' unless %{$element} ;
 			last ;
 			} ;
 			
@@ -415,6 +426,7 @@ for (my $nodes_left = $#nodes_to_display ; $nodes_left >= 0 ; $nodes_left--)
 				
 			$tag = 'A' ;
 			$perl_address = "$element" if($setup->{DISPLAY_PERL_ADDRESS}) ;
+			$default_element_rendering = ' (no elements)' unless @{$element} ;
 			last ;
 			} ;
 			
@@ -423,6 +435,7 @@ for (my $nodes_left = $#nodes_to_display ; $nodes_left >= 0 ; $nodes_left--)
 			$is_terminal_node++ ;
 			$tag = 'C' ;
 			$element_value = "$element" ;
+			$default_element_rendering= " = $element_value" ;
 			$perl_address = "$element_id" if($setup->{DISPLAY_PERL_ADDRESS}) ;
 			last ;
 			} ;
@@ -453,6 +466,7 @@ for (my $nodes_left = $#nodes_to_display ; $nodes_left >= 0 ; $nodes_left--)
 		# DEFAULT, an object.
 		$tag = 'O' ;
 		$element_value = "Object of type '" . ref($element) . "'" if($setup->{DISPLAY_OBJECT_TYPE}) ;
+		$default_element_rendering = " = $element_value" if($setup->{DISPLAY_OBJECT_TYPE}) ;
 		$perl_address = "$element" if($setup->{DISPLAY_PERL_ADDRESS}) ;
 		}
 		
@@ -515,6 +529,8 @@ for (my $nodes_left = $#nodes_to_display ; $nodes_left >= 0 ; $nodes_left--)
 			}
 		else
 			{
+			# build in renderer
+			
 			#--------------------------
 			# wrapping	
 			#--------------------------
@@ -522,19 +538,7 @@ for (my $nodes_left = $#nodes_to_display ; $nodes_left >= 0 ; $nodes_left--)
 			my $tree_header            = $setup->{INDENTATION} . $level_text . $previous_level_separator . $separator  ;
 			my $tree_subsequent_header = $setup->{INDENTATION} . $level_text . $previous_level_separator . $subsequent_separator ;
 			
-			my $element_description = $element_name ;
-			
-			if('S' eq $tag)
-				{
-				if($setup->{QUOTE_VALUES})
-					{
-					$element_description .= " = '$element_value'" ;
-					}
-				else
-					{
-					$element_description .= " = $element_value" ;
-					}
-				}
+			my $element_description = $element_name . $default_element_rendering ;
 			
 			$perl_size = " <$perl_size> " unless $perl_size eq '' ;
 			
@@ -891,7 +895,9 @@ return
 sub GetCallerStack
 {
 my $level_to_dump = shift || 1_000_000;
-my $current_level = 1 ; # skip this function 
+my $current_level = 2 ; # skip this function 
+
+$level_to_dump += $current_level ; #
 
 my @stack_dump ;
 
@@ -922,7 +928,7 @@ while ($current_level < $level_to_dump)
 	#~ , 'Stack dump:'
 	#~ , DISPLAY_ADDRESS => 1
 	#~ , MAX_DEPTH => 5
-	#~ , DISPLAY_OBJECT_TYPE => 0
+	#~ , DISPLAY_OBJECT_TYPE => 1
 	#~ , USE_ASCII => 1
 	#~ , QUOTE_VALUES => 1
 	#~ ) ;
@@ -1155,8 +1161,6 @@ the scalar values.
 
   DumpTree(\$s, 'Cells:', QUOTE_VALUES=> 1) ;
   
-  # output
-
 =head2 NO_OUTPUT
 
 This option suppresses all output generated by Data::TreeDumper. 
